'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var jstdgen = require('json-schema-test-data-generator');
var _ = require('lodash');
var ps = require('prop-search');
var deref = require('json-schema-deref');

function getLinks(schema) {
  var schemaLinksRes = ps.searchForExistence(schema, 'links', { separator: '.' });
  var schemaLinks = [];
  if (schemaLinksRes && schemaLinksRes.length > 0) {
    schemaLinks = _.chain(schemaLinksRes).map(function (link) {
      return link.value;
    }).map('links').flatten(true).compact().filter(function (link) {
      return link.schema || link.targetSchema;
    }).value();
  }

  return schemaLinks;
}

function generateData(schema, fn) {
  var links = getLinks(schema);
  if (!links || !links.length) {
    return fn(null, []);
  }

  var ret = [];
  for (var i = 0; i < links.length; i++) {
    var _data$data;

    var link = links[i];

    if (!link) {
      continue;
    }

    var data = {
      link: link,
      data: []
    };

    if (typeof link.method === 'string' && typeof link.rel === 'string') {
      var method = link.method.toUpperCase();
      var rel = link.rel.toLowerCase();
      if (method === 'GET') {
        if (rel === 'self') {
          data.data.push({
            message: 'should get instance'
          });
          data.data.push({
            message: 'should get 404 on unknown instance'
          });
        } else if (rel === 'instances') {
          data.data.push({
            message: 'should get empty list of instances if none exist'
          });
          data.data.push({
            message: 'should get list of instances'
          });
        }
      } else if (method === 'DELETE') {
        data.data.push({
          message: 'should delete instance'
        });
        data.data.push({
          message: 'should get 404 on unknown instance'
        });
      }
    }

    if (!link.schema) {
      ret.push(data);
      continue;
    }

    (_data$data = data.data).push.apply(_data$data, _toConsumableArray(jstdgen(link.schema)));
    ret.push(data);
  }

  return fn(null, ret);
}

/**
 * Generate sample test data based on links in JSON Hyper-Schema definition
 * @param  {Object}   schema  JSON Hyper-Schema definition
 * @param  {Object}   options The options. For additional schema deref options See {@link https://www.npmjs.com/package/json-schema-deref json-schema-deref}
 * @param  {Boolean}  options.skipDeref If you are sure that there are no <code>$ref</code>'s in your schema, use this
 *                                      to skip schema deref, and it will make things faster. Default: <code>false</code>.
 * @param  {Function} fn      Callback in standard form <code>(err, result)</code>
 */
function generate(schema, options, fn) {
  if (_.isFunction(options)) {
    fn = options;
    options = {};
  }

  options = _.defaults(options, {
    skipDeref: false
  });

  if ((typeof schema === 'undefined' ? 'undefined' : _typeof(schema)) !== 'object') {
    process.nextTick(function () {
      return fn(new Error('No schema specified'));
    });
  }

  if (options.skipDeref) {
    process.nextTick(function () {
      return generateData(schema, fn);
    });
  }

  deref(schema, options, function (err, fullSchema) {
    if (err) {
      return fn(err);
    }

    generateData(fullSchema, fn);
  });
}

module.exports = generate;